#include "raduino_sbus.h"
#include <Arduino.h>

void SBUS::begin() {
	SBUS::begin(true);
  SBUS_OUT = false;
}

void SBUS::begin(bool useTimer) {
//	if (useTimer) {
//		noInterrupts();
//		TCCR2A  = 0;
//		TCCR2B  = 0;
//		TCNT2   = 0;
//		OCR2A   = 249;
//		TCCR2A |= (1 << WGM21);
//		TCCR2B |= (1 << CS22);
//		TIMSK2 |= (1 << OCIE2A);
//		interrupts();
//	}

	for (byte i = 0; i<18; i++) {
		_channels[i]      = 0;
	}

	_goodFrames         = 0;
	_lostFrames         = 0;
	_decoderErrorFrames = 0;
	_failsafe           = SBUS_FAILSAFE_INACTIVE;

 for (int i = 0; i<18; i++) {
  ch1.channel[i] =0;
  ch1.value[i]=0;
 }
	
	_serial.begin(100000, SERIAL_8E2);
}

void SBUS::process() {
	static byte buffer[25];
	static byte buffer_index = 0;

	while (_serial.available()) {
		byte rx = _serial.read();
    //_serial.write(rx);                                     // 2018.03.20 DNU
		if (buffer_index == 0 && rx != SBUS_STARTBYTE) {
			//incorrect start byte, out of sync
			_decoderErrorFrames++;
			continue;
		}

		buffer[buffer_index++] = rx;

		if (buffer_index == 25) {
			buffer_index = 0;
			if (buffer[24] != SBUS_ENDBYTE) {
				//incorrect end byte, out of sync
				_decoderErrorFrames++;
				continue;
			}
			_goodFrames++;

			_channels[0]  = ((buffer[1]    |buffer[2]<<8)                 & 0x07FF);
			_channels[1]  = ((buffer[2]>>3 |buffer[3]<<5)                 & 0x07FF);
			_channels[2]  = ((buffer[3]>>6 |buffer[4]<<2 |buffer[5]<<10)  & 0x07FF);
			_channels[3]  = ((buffer[5]>>1 |buffer[6]<<7)                 & 0x07FF);
			_channels[4]  = ((buffer[6]>>4 |buffer[7]<<4)                 & 0x07FF);
			_channels[5]  = ((buffer[7]>>7 |buffer[8]<<1 |buffer[9]<<9)   & 0x07FF);
			_channels[6]  = ((buffer[9]>>2 |buffer[10]<<6)                & 0x07FF);
			_channels[7]  = ((buffer[10]>>5|buffer[11]<<3)                & 0x07FF);
			_channels[8]  = ((buffer[12]   |buffer[13]<<8)                & 0x07FF);
			_channels[9]  = ((buffer[13]>>3|buffer[14]<<5)                & 0x07FF);
			_channels[10] = ((buffer[14]>>6|buffer[15]<<2|buffer[16]<<10) & 0x07FF);
			_channels[11] = ((buffer[16]>>1|buffer[17]<<7)                & 0x07FF);
			_channels[12] = ((buffer[17]>>4|buffer[18]<<4)                & 0x07FF);
			_channels[13] = ((buffer[18]>>7|buffer[19]<<1|buffer[20]<<9)  & 0x07FF);
			_channels[14] = ((buffer[20]>>2|buffer[21]<<6)                & 0x07FF);
			_channels[15] = ((buffer[21]>>5|buffer[22]<<3)                & 0x07FF);

			((buffer[23])      & 0x0001) ? _channels[16] = 2047: _channels[16] = 0;
			((buffer[23] >> 1) & 0x0001) ? _channels[17] = 2047: _channels[17] = 0;

			if ((buffer[23] >> 3) & 0x0001) {
				_failsafe = SBUS_FAILSAFE_ACTIVE;
			} else {
				_failsafe = SBUS_FAILSAFE_INACTIVE;
			}

			if ((buffer[23] >> 2) & 0x0001) {
				_lostFrames++;
			}

      if(SBUS_OUT == true){//SBUS_OUT이 true일때
          //채널값을 변경한다. 매개변수 채널번호, 채널 값, buffer
          writeChannel( &buffer[0]);
          //SBUS OUT으로 가공된 데이타를 보낸다.
          for(int i = 0; i<25; i++){                          
            _serial.write(buffer[i]);
          }
      }
			_lastGoodFrame = millis();
		}
	}
}

int SBUS::getChannel(int channel) {
	if (channel < 1 or channel > 18) {
		return 0;
	} else {
		return _channels[channel - 1];
	}
}

int SBUS::getNormalizedChannel(int channel) {
	if (channel < 1 or channel > 18) {
		return 0;
	} else {
		return (int) lround(_channels[channel - 1] / 9.92) - 100; //9.92 or 10.24?
	}
}

int SBUS::getFailsafeStatus() {
	return _failsafe;
}

int SBUS::getFrameLoss() {
	return (int) ((_lostFrames + _decoderErrorFrames) * 100 / (_goodFrames + _lostFrames + _decoderErrorFrames));
}

long SBUS::getGoodFrames() {
	return _goodFrames;
}

long SBUS::getLostFrames() {
	return _lostFrames;
}

long SBUS::getDecoderErrorFrames() {
	return _decoderErrorFrames;
}

long long SBUS::getLastTime() {
	return _lastGoodFrame;
}

void SBUS::setChannel(int channel, unsigned int value) {
  ch1.channel[channel] = 1; //ch1 : selected
  ch1.value[channel]   = value; //value : 0
}

void SBUS::writeChannel(byte *buffers){

  for (int i = 1; i<19; i++) {
    if(ch1.channel[i] > 0)
    {    

      switch(i-1)
      {
        case 0:
    //      _channels[0]  = ((buffers[1]    |buffers[2]<<8)                 & 0x07FF);
          buffers[1] = (buffers[1] & 0x00) | byte((ch1.value[i] & 0x07FF));
          buffers[2] = (buffers[2] & 0xF8) | byte((ch1.value[i] & 0x07FF)>>8);
          break;
        case 1:
    //      _channels[1]  = ((buffers[2]>>3 |buffers[3]<<5)                 & 0x07FF);
          buffers[2] = (buffers[2] & 0x07) | byte((ch1.value[i] & 0x07FF)<<3);
          buffers[3] = (buffers[3] & 0xC0) | byte((ch1.value[i] & 0x07FF)>>5);
          break;
        case 2:
    //      _channels[2]  = ((buffers[3]>>6 |buffers[4]<<2 |buffers[5]<<10)  & 0x07FF);
          buffers[3] = (buffers[3] &0x3F)| byte((ch1.value[i] & 0x07FF)<<6);
          buffers[4] = (buffers[4] &0x00)| byte((ch1.value[i] & 0x07FF)>>2);
          buffers[5] = (buffers[5] &0xFE)| byte((ch1.value[i] & 0x07FF)>>10);
          break;
        case 3:
    //      _channels[3]  = ((buffers[5]>>1 |buffers[6]<<7)                 & 0x07FF);
          buffers[5] = (buffers[5] & 0x01) | byte((ch1.value[i] & 0x07FF)<<1);
          buffers[6] = (buffers[6] & 0xF0) | byte((ch1.value[i] & 0x07FF)>>7);
          break;
        case 4:
    //      _channels[4]  = ((buffers[6]>>4 |buffers[7]<<4)                 & 0x07FF);
          buffers[6] = (buffers[6] & 0x0F) | byte((ch1.value[i] & 0x07FF)<<4);
          buffers[7] = (buffers[7] & 0x80) | byte((ch1.value[i] & 0x07FF)>>4);
          break;
        case 5:
    //      _channels[5]  = ((buffers[7]>>7 |buffers[8]<<1 |buffers[9]<<9)   & 0x07FF);
          buffers[7] = (buffers[7] & 0x7F) | byte((ch1.value[i] & 0x07FF)<<7);
          buffers[8] = (buffers[8] & 0x00) | byte((ch1.value[i] & 0x07FF)>>1);
          buffers[9] = (buffers[9] & 0xFC) | byte((ch1.value[i] & 0x07FF)>>9);
          break;
        case 6:
    //      _channels[6]  = ((buffers[9]>>2 |buffers[10]<<6)                & 0x07FF);
          buffers[9] = (buffers[9] & 0x03) | byte((ch1.value[i] & 0x07FF)<<2);
          buffers[10] = (buffers[10] & 0xE0) | byte((ch1.value[i] & 0x07FF)>>6);
          break;
        case 7:
    //      _channels[7]  = ((buffers[10]>>5|buffers[11]<<3)                & 0x07FF);
          buffers[10] = (buffers[10] & 0x1F) | byte((ch1.value[i] & 0x07FF)<<5);
          buffers[11] = (buffers[11] & 0x00) | byte((ch1.value[i] & 0x07FF)>>3);
          break;
        case 8:
    //      _channels[8]  = ((buffers[12]   |buffers[13]<<8)                & 0x07FF);
          buffers[12] = (buffers[12] & 0x00) | byte((ch1.value[i] & 0x07FF));
          buffers[13] = (buffers[13] & 0xF8) | byte((ch1.value[i] & 0x07FF)>>8);
          break;
        case 9:
    //      _channels[9]  = ((buffers[13]>>3|buffers[14]<<5)                & 0x07FF);    
          buffers[13] = (buffers[13] & 0x07) | byte((ch1.value[i] & 0x07FF)<<3);
          buffers[14] = (buffers[24] & 0xC0) | byte((ch1.value[i] & 0x07FF)>>5);
          break;
        case 10:
    //      _channels[10] = ((buffers[14]>>6|buffers[15]<<2|buffers[16]<<10) & 0x07FF);
          buffers[14] = (buffers[14] & 0x3F) | byte((ch1.value[i] & 0x07FF)<<6);
          buffers[15] = (buffers[15] & 0x00) | byte((ch1.value[i] & 0x07FF)>>2);
          buffers[16] = (buffers[16] & 0xFE) | byte((ch1.value[i] & 0x07FF)>>10);
          break;
        case 11:
    //      _channels[11] = ((buffers[16]>>1|buffers[17]<<7)                & 0x07FF);
          buffers[16] = (buffers[16] & 0x01) | byte((ch1.value[i] & 0x07FF)<<1);
          buffers[17] = (buffers[17] & 0xF0) | byte((ch1.value[i] & 0x07FF)>>7);
          break;
        case 12:
    //     _channels[12] = ((buffers[17]>>4|buffers[18]<<4)                & 0x07FF);
          buffers[17] = (buffers[17] & 0x0F) | byte((ch1.value[i] & 0x07FF)<<4);
          buffers[18] = (buffers[18] & 0x80) | byte((ch1.value[i] & 0x07FF)>>4);
          break;
        case 13:
    //     _channels[13] = ((buffers[18]>>7|buffers[19]<<1|buffers[20]<<9)  & 0x07FF);
          buffers[18] = (buffers[18] & 0x7F) | byte((ch1.value[i] & 0x07FF)<<7);
          buffers[19] = (buffers[19] & 0x00) | byte((ch1.value[i] & 0x07FF)>>1);
          buffers[20] = (buffers[20] & 0xFC) | byte((ch1.value[i] & 0x07FF)>>9);
          break;
        case 14:
    //    _channels[14] = ((buffers[20]>>2|buffers[21]<<6)                & 0x07FF);
          buffers[20] = (buffers[20] & 0x03) | byte((ch1.value[i] & 0x07FF)<<2);
          buffers[21] = (buffers[21] & 0xE0) | byte((ch1.value[i] & 0x07FF)>>6);
          break;
        case 15:
    //    _channels[15] = ((buffers[21]>>5|buffers[22]<<3)                & 0x07FF);
          buffers[21] = (buffers[21] & 0x1F) | byte((ch1.value[i] & 0x07FF)<<5);
          buffers[22] = (buffers[22] & 0x00) | byte((ch1.value[i] & 0x07FF)>>3);
          break;     
        default:
          break;
      } 
    }   
  }        
}
