#include "raduino_fc.h"
#include <Arduino.h>

void FC::begin() {

	for (byte i = 0; i<11; i++) {
		RxValue[i]      = 0;
	}

  inputString = "";         // a String to hold incoming data
  retString = "";

  // reserve 200 bytes for the inputString:
  inputString.reserve(200);
  retString.reserve(200);
    
  stringComplete = false;  // whether the string is complete
	
	_serial.begin(115200);
}


int FC::Event() {
  int ret=0;
  while (_serial.available()) {
    // get the new byte:
    char inChar = (char)_serial.read();
    // add it to the inputString:
    inputString += inChar;
    // if the incoming character is a newline, set a flag so the main loop can
    // do something about it:
    if (inChar == '\n') {
      stringComplete = true;
    }
  }
  
  if (stringComplete) {

    // clear the string:
    retString = inputString;
     
    if(inputString=="EX_BIND\n") ret=1; //ex_bind
    else if(inputString=="RX_BIND\n") ret=2; //rx_bind    
    else ret = 10;
    
    inputString = "";
    stringComplete = false;
  }
  return ret;
}
  
void FC::Tx(int roll, int pitch, int yaw, int throttle) {  

  RxValue[2] = roll/256;
  RxValue[3] = roll%256;
  RxValue[4] = pitch/256;
  RxValue[5] = pitch%256;
  RxValue[6] = yaw/256;
  RxValue[7] = yaw%256;    
  RxValue[8] = throttle/256;
  RxValue[9] = throttle%256;
  
  RxValue[0] = 'U';
  RxValue[1] = 'N';
  RxValue[10] = 'C';
  for(int i = 0 ; i < 11; i++)
  {
    _serial.write(RxValue[i]); 
  }
}

void FC::Bind() { //bind
  RxValue[0] = 'B';
  RxValue[1] = 'I';
  RxValue[2] = 'N';
  RxValue[3] = 'D';
  for(int i = 0 ; i < 11; i++)
  {
    _serial.write(RxValue[i]); 
  }
}

//Calibration gyro and acc
void FC::Cali() { 
  RxValue[0] = 'C';
  RxValue[1] = 'A';
  RxValue[2] = 'L';
  RxValue[3] = 'I';
  for(int i = 0 ; i < 11; i++)
  {
    _serial.write(RxValue[i]); 
  }
}

// pitch positive forward
// roll positive right
void FC::Trim(int roll, int pitch) { //trim roll, pitch
  RxValue[0] = 'T';
  RxValue[1] = 'R';
  RxValue[2] = 'I';
  RxValue[3] = 'M';
  RxValue[10] = 'C';

  if(roll < 0) RxValue[4]='-';
  else        RxValue[4]='+';
  
  RxValue[5] = abs(roll)/256;
  RxValue[6] = abs(roll)%256;

  if(pitch < 0) RxValue[7]='-';
  else         RxValue[7]='+';

  RxValue[8] = abs(pitch)/256;
  RxValue[9] = abs(pitch)%256;
  
  for(int i = 0 ; i < 11; i++)
  {
    _serial.write(RxValue[i]); 
  }
}

//Set PID kp
void FC::SetPIDkp(int roll, int pitch, int yaw) { // roll, pitch
  RxValue[0] = 'P';
  RxValue[1] = 'I';
  RxValue[2] = 'D';
  RxValue[3] = 'P';
  RxValue[10] = 'C';
 
  RxValue[4] = abs(roll)/256;
  RxValue[5] = abs(roll)%256;

  RxValue[6] = abs(pitch)/256;
  RxValue[7] = abs(pitch)%256;
  
  RxValue[8] = abs(yaw)/256;
  RxValue[9] = abs(yaw)%256;

  for(int i = 0 ; i < 11; i++)
  {
    _serial.write(RxValue[i]); 
  }
}

//Set PID ki
void FC::SetPIDki(int roll, int pitch, int yaw) { // roll, pitch
  RxValue[0] = 'P';
  RxValue[1] = 'I';
  RxValue[2] = 'D';
  RxValue[3] = 'I';
  RxValue[10] = 'C';
 
  RxValue[4] = abs(roll)/256;
  RxValue[5] = abs(roll)%256;

  RxValue[6] = abs(pitch)/256;
  RxValue[7] = abs(pitch)%256;
  
  RxValue[8] = abs(yaw)/256;
  RxValue[9] = abs(yaw)%256;

  for(int i = 0 ; i < 11; i++)
  {
    _serial.write(RxValue[i]); 
  }
}

//Set PID kd
void FC::SetPIDkd(int roll, int pitch, int yaw) { // roll, pitch
  RxValue[0] = 'P';
  RxValue[1] = 'I';
  RxValue[2] = 'D';
  RxValue[3] = 'D';
  RxValue[10] = 'C';
 
  RxValue[4] = abs(roll)/256;
  RxValue[5] = abs(roll)%256;

  RxValue[6] = abs(pitch)/256;
  RxValue[7] = abs(pitch)%256;
  
  RxValue[8] = abs(yaw)/256;
  RxValue[9] = abs(yaw)%256;

  for(int i = 0 ; i < 11; i++)
  {
    _serial.write(RxValue[i]); 
  }
}
